# Copyright (c) IBM Corporation

# SPDX-License-Identifier: MIT

import logging

import pandas as pd

# Configure logging

logging.basicConfig(
    level=logging.INFO,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",  # Log format
    handlers=[logging.StreamHandler()],  # Output to console
)

logger = logging.getLogger(__name__)


def to_series(x: pd.Series | pd.DataFrame | dict) -> pd.Series:
    if isinstance(x, pd.Series):
        return x

    if isinstance(x, pd.DataFrame):
        if len(x) != 1:
            raise ValueError(f"DataFrame must have exactly 1 row, got {len(x)}")
        return x.iloc[0]

    if isinstance(x, dict):
        s = pd.Series(x)
        if s.empty:
            raise ValueError("Config from dict cannot be empty")
        return s

    raise TypeError(f"Expected Series, DataFrame, or dict, got {type(x).__name__}")


def is_row_valid(
    config: pd.Series | pd.DataFrame | dict,
    err_prefix: str = "Rule-based classifier error: ",
) -> tuple[bool, list[str]]:
    """
    Applies to rows a rule-based classification
    """
    errors = []
    config = to_series(config)

    # Rule 1
    try:
        if config["batch_size"] % config["number_gpus"] != 0:
            errors.append(
                err_prefix + "batch_size must be evenly divisible by number_gpus."
            )
    except Exception as e:
        except_string = """Rule based on divisibility of 'batch_size' by 'number_gpus'
                    cannot be applied, probably because number_gpus has not been specified on the config"""
        logger.info(f"{e}:{except_string}")
        return False, errors

    return len(errors) == 0, errors



# Copyright (c) IBM Corporation

# SPDX-License-Identifier: MIT

"""
Module for mapping between categorical variable values in the production data to the training data derived from benchmarking.
"""

# TODO(srikumarv): Need to make this formal somewhere so that it is resilient and adaptable to model and training data updates

import re

model_patterns = {
    "GRANITE_3_1_2B": r"^granite-(?:3\.[1-3]-)?2b(?:-(?:instruct|base))?(?:-|$)",
    "GRANITE_3_1_8B": r"^granite-(?:3\.[1-3]-)?8b(?:-(?:instruct|base))?$",
    "LLAMA_3_1_8B": r"llama-3.1-8b(?:-.*)?$",
    "GRANITE_4_SMALL": r"granite-4\.0(?:-h)?-(?:small)(?:-|$)",
    "GRANITE_4_TINY": r"granite-4\.0(?:-h)?-(?:tiny)(?:-|$)",
    "GRANITE_4_MICRO": r"granite-4\.0(?:-h)?-(?:micro)(?:-|$)",
}

# Models that we have chosen to map to

mapped_models = {
    "GRANITE_3_1_2B": "granite-3.1-2b",
    "GRANITE_3_1_8B": "granite-3.1-8b-instruct",
    "LLAMA_3_1_8B": "llama-3.1-8b",
    "GRANITE_4_SMALL": "granite-4.0-h-small",
    "GRANITE_4_TINY": "granite-4.0-h-tiny",
    "GRANITE_4_MICRO": "granite-4.0-h-micro",
}


def map_valid_model_name(model_name: str) -> str:

    mapped = [
        mapped_models[k] for k, v in model_patterns.items() if re.match(v, model_name)
    ]
    return model_name if len(mapped) == 0 else mapped[0]


def map_to_valid(input_dict: dict) -> dict:
    from copy import deepcopy

    mapped_dict = deepcopy(input_dict)
    key = "model_name"
    if key in mapped_dict:
        mapped_dict[key] = map_valid_model_name(mapped_dict[key])

    key = "gpu_model"
    if mapped_dict.get(key, None) == "NVIDIA A100-SXM4-80GB":
        mapped_dict[key] = "NVIDIA-A100-SXM4-80GB"

    return mapped_dict



# Copyright (c) IBM Corporation

# SPDX-License-Identifier: MIT

from autoconf.utils.config_mapper import (
    map_valid_model_name,
    mapped_models,
)

invalid_model_names_transform_to_valid = {
    "granite-2b-base": mapped_models["GRANITE_3_1_2B"],
    "granite-3.2-8b-instruct": mapped_models["GRANITE_3_1_8B"],
    "granite-4.0-tiny": mapped_models["GRANITE_4_TINY"],
    "granite-4.0-small-base-prerelease-greylock": mapped_models["GRANITE_4_SMALL"],
    "llama-3-1-8b-instruct": mapped_models["LLAMA_3_1_8B"],
}

invalid_model_names_no_transform = {
    "gb_tuned_model_fw0u3wim_checkpoint-32637": "gb_tuned_model_fw0u3wim_checkpoint-32637",
    "granite-8b-code-instruct-128k": "granite-8b-code-instruct-128k",
    "devstral-small-182a9e3_24b": "devstral-small-182a9e3_24b",
    "granite-5.0-d453da": "granite-5.0-d453da",
}

valid_model_names_no_transform = {
    mapped_models["LLAMA_3_1_8B"]: mapped_models["LLAMA_3_1_8B"],
    mapped_models["GRANITE_4_TINY"]: mapped_models["GRANITE_4_TINY"],
    mapped_models["GRANITE_4_MICRO"]: mapped_models["GRANITE_4_MICRO"],
    mapped_models["GRANITE_3_1_8B"]: mapped_models["GRANITE_3_1_8B"],
}


def test_mapping_invalid_to_valid():
    assert all(
        map_valid_model_name(k) == v
        for k, v in invalid_model_names_transform_to_valid.items()
    )


def test_mapping_invalid_no_transform():
    assert all(
        map_valid_model_name(k) == v
        for k, v in invalid_model_names_no_transform.items()
    )


def test_mapping_valid_no_transform():
    assert all(
        map_valid_model_name(k) == v for k, v in valid_model_names_no_transform.items()
    )



# Copyright (c) IBM Corporation

# SPDX-License-Identifier: MIT

from unittest.mock import MagicMock, patch

import pandas as pd
import pytest

from autoconf.utils.pydantic_models import JobConfig
from autoconf.utils.recommender import (
    get_model_prediction_and_metadata,
    recommend_min_gpu,
)

# Example configurations

valid_config_dict = {
    "model_name": "granite-3.2-8b-instruct",
    "method": "lora",
    "gpu_model": "NVIDIA-A100-80GB-PCIe",
    "tokens_per_sample": 8192.0,
    "batch_size": 16.0,
    "number_gpus": 2,
    "model_version": "2.0.0",
}

invalid_config_dict = {
    "model_name": "llama-13b",
    "method": "full",
    "gpu_model": "NVIDIA-A100-80GB-PCIe",
    "tokens_per_sample": 4096.0,
    "batch_size": 128.0,
    "number_gpus": 8,
    "model_version": "2.0.0",
}

# Convert to JobConfig instances

valid_job_config = JobConfig(**valid_config_dict)
invalid_job_config = JobConfig(**invalid_config_dict)

# Mock return values

mock_prediction_valid = [1]
mock_prediction_invalid = [0]
mock_rbc_valid = (1, [])
mock_rbc_invalid = (0, ["Invalid configuration"])


@pytest.fixture
def mock_predictor_valid():
    mock = MagicMock()

    def mock_predict(df):
        # Simulate prediction logic based on gpus_per_worker
        print(df)
        val = 1 if int(df["number_gpus"].values[0]) == 2 else 0
        return pd.Series([val])  # Mimics .values[0] behavior

    mock.predict.side_effect = mock_predict
    return mock


@pytest.fixture
def mock_predictor_invalid():
    mock = MagicMock()

    def mock_predict(df):
        val = -1
        return pd.Series([val])

    mock.predict.side_effect = mock_predict
    return mock


@patch(
    "autoconf.utils.rule_based_classifier.is_row_valid",
    return_value=mock_rbc_valid,
)
def test_get_model_prediction_and_metadata_valid(
    mock_is_row_valid, mock_predictor_valid
):
    print(valid_job_config)
    pred, metadata = get_model_prediction_and_metadata(
        valid_job_config, predictor=mock_predictor_valid
    )
    assert pred == 1
    assert isinstance(metadata, dict)
    assert metadata["Rule-Based Classifier error"] == ""


@patch(
    "autoconf.utils.rule_based_classifier.is_row_valid",
    return_value=mock_rbc_invalid,
)
def test_get_model_prediction_and_metadata_invalid(
    mock_is_row_valid, mock_predictor_valid
):
    pred, metadata = get_model_prediction_and_metadata(
        invalid_job_config, predictor=mock_predictor_valid
    )
    assert pred is None or pred == 0
    assert isinstance(metadata, dict)


@patch(
    "autoconf.utils.rule_based_classifier.is_row_valid",
    return_value=mock_rbc_valid,
)
def test_recommend_min_gpu_valid(mock_is_row_valid, mock_predictor_valid):
    min_gpu, metadata = recommend_min_gpu(
        valid_job_config, valid_n_gpu_list=[1, 2, 4], predictor=mock_predictor_valid
    )
    assert min_gpu == 2
    assert isinstance(metadata, dict)


@patch(
    "autoconf.utils.rule_based_classifier.is_row_valid",
    return_value=mock_rbc_invalid,
)
def test_recommend_min_gpu_invalid(mock_is_row_valid, mock_predictor_invalid):
    min_gpu, metadata = recommend_min_gpu(
        invalid_job_config, valid_n_gpu_list=[1, 2, 4], predictor=mock_predictor_invalid
    )
    assert min_gpu == -1
    assert isinstance(metadata, dict)



# Copyright (c) IBM Corporation

# SPDX-License-Identifier: MIT

import functools
import importlib.resources
import logging
import math
import traceback
from typing import NamedTuple

from autogluon.tabular import TabularPredictor

from autoconf.utils.pydantic_models import JobConfig
from autoconf.utils.recommender import (
    NoRecommendationError,
    recommend_min_gpu,
)
from orchestrator.modules.actuators.custom_experiments import custom_experiment
from orchestrator.schema.domain import PropertyDomain, VariableTypeEnum
from orchestrator.schema.property import ConstitutiveProperty

moduleLog = logging.getLogger()


class GPUsAndWorkers(NamedTuple):
    gpus: int
    workers: int


@functools.cache
def load_model(model_version: str) -> TabularPredictor:
    """Loads the model

    Args:
        model_version:
            The version of the Autogluon model to use e.g. 1.0.0

    Returns:
        The predictor
    """
    if model_version == "1.1.0":
        path_weights = str(
            importlib.resources.files("autoconf")
            / "AutoGluonModels"
            / "v1-1-0_ag-20251112_155927-refit-clone-opt"
        )
    elif model_version == "2.0.0":
        path_weights = str(
            importlib.resources.files("autoconf")
            / "AutoGluonModels"
            / "v2-0-0_ag-20251113_154241-refit-clone-opt"
        )
    else:
        raise ValueError("Unknown model_version", model_version)

    return TabularPredictor.load(path_weights, require_py_version_match=False)


ModelVersion = ConstitutiveProperty(
    identifier="model_version",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CATEGORICAL_VARIABLE_TYPE,
        values=["1.1.0", "2.0.0"],
    ),
)

ModelName = ConstitutiveProperty(
    identifier="model_name",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CATEGORICAL_VARIABLE_TYPE,
        values=[
            "allam-1-13b",
            "granite-13b-v2",
            "granite-20b-v2",
            "granite-3-8b",
            "granite-3.1-2b",
            "granite-3.1-3b-a800m-instruct",
            "granite-3.1-8b-instruct",
            "granite-34b-code-base",
            "granite-3b-code-base-128k",
            "granite-4.0-1b",
            "granite-4.0-350m",
            "granite-4.0-h-1b",
            "granite-4.0-h-micro",
            "granite-4.0-h-small",
            "granite-4.0-h-tiny",
            "granite-4.0-micro",
            "granite-7b-base",
            "granite-8b-code-base",
            "granite-8b-japanese",
            "llama-13b",
            "llama-7b",
            "llama2-70b",
            "llama3-70b",
            "llama3-8b",
            "llama3.1-405b",
            "llama3.1-70b",
            "llama3.1-8b",
            "mistral-123b-v2",
            "mistral-7b-v0.1",
            "mixtral-8x7b-instruct-v0.1",
        ],
    ),
)

TuningMethod = ConstitutiveProperty(
    identifier="method",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CATEGORICAL_VARIABLE_TYPE, values=["full", "lora"]
    ),
)

GPUModel = ConstitutiveProperty(
    identifier="gpu_model",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CATEGORICAL_VARIABLE_TYPE,
        values=[
            "L40S",
            "NVIDIA-A100-80GB-PCIe",
            "NVIDIA-A100-SXM4-80GB",
            "NVIDIA-H100-PCIe",
        ],
    ),
)

TokensPerSample = ConstitutiveProperty(
    identifier="tokens_per_sample",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CONTINUOUS_VARIABLE_TYPE,
        domainRange=[1, 10000001],  # VV: Seen values go up to 8192
    ),
)

BatchSize = ConstitutiveProperty(
    identifier="batch_size",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CONTINUOUS_VARIABLE_TYPE,
        domainRange=[1, 10000001],  # VV: Seen values go up to 128
    ),
)
GPUsPerWorker = ConstitutiveProperty(
    identifier="gpus_per_worker",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CONTINUOUS_VARIABLE_TYPE,
        domainRange=[1, 10000001],  # VV: Seen values is just [8]
    ),
)

MaxGPUs = ConstitutiveProperty(
    identifier="max_gpus",
    propertyDomain=PropertyDomain(
        variableType=VariableTypeEnum.CONTINUOUS_VARIABLE_TYPE,
        domainRange=[1, 10000001],  # VV: Seen values go up to 128
    ),
)


@custom_experiment(
    required_properties=[
        ModelName,
        TuningMethod,
        GPUModel,
        TokensPerSample,
        BatchSize,
        ModelVersion,
    ],
    optional_properties=[GPUsPerWorker, MaxGPUs],
    output_property_identifiers=["can_recommend", "gpus", "workers"],
    metadata={
        "description": "An AutoConf plugin that suggests the minimum number of "
        "gpus per worker and number of workers necessary to execute a Tuning job"
    },
    parameterization={},
)
def min_gpu_recommender(
    model_name: str,
    method: str,
    gpu_model: str,
    tokens_per_sample: int,
    batch_size: int,
    model_version: str,
    gpus_per_worker: int = 8,
    max_gpus: int = 8,
) -> dict[str, int | bool]:
    try:
        parameters = {
            "model_name": model_name,
            "method": method,
            "gpu_model": gpu_model,
            "tokens_per_sample": tokens_per_sample,
            "batch_size": batch_size,
            "model_version": model_version,
            "gpus_per_worker": gpus_per_worker,
            "max_gpus": max_gpus,
        }
        try:
            predictor = load_model(model_version=model_version)

            config = JobConfig.model_validate(
                {
                    "model_name": parameters["model_name"],
                    "gpu_model": parameters["gpu_model"],
                    "method": parameters["method"],
                    "tokens_per_sample": parameters["tokens_per_sample"],
                    "batch_size": parameters["batch_size"],
                }
            )
            moduleLog.debug(f"Configuration supplied is {config}")
            valid_n_gpus = []
            i = 1
            while i <= max_gpus:
                valid_n_gpus.append(i)
                i *= 2

            min_gpus, metadata = recommend_min_gpu(
                config, predictor=predictor, valid_n_gpu_list=valid_n_gpus
            )

            if min_gpus < 1:
                raise NoRecommendationError(str(metadata))

            workers = math.ceil(min_gpus / gpus_per_worker)
            gpus = math.ceil(min_gpus / workers)

            ret = GPUsAndWorkers(gpus=gpus, workers=workers)
        except NoRecommendationError as e:
            moduleLog.warning(
                f"recommend_min_gpus_and_workers() for {parameters} cannot produce a recommendation: {e}"
            )
            return {"can_recommend": False}
        except ValueError as e:
            # Handling the case when the validation of pydantic model fails
            moduleLog.warning(
                f"recommend_min_gpus_and_workers() for {parameters} failed with error {e}"
            )
            moduleLog.debug(f"Traceback {traceback.format_exc()}")
            return {"can_recommend": False}

        return {
            "can_recommend": True,
            "gpus": ret.gpus,
            "workers": ret.workers,
        }
    except Exception as e:
        # General failure due to recommender model not loading.. autogluon environment issues
        # should result in InvalidMeasurements
        moduleLog.warning(e)
        raise e



# Copyright (c) IBM Corporation

# SPDX-License-Identifier: MIT

import logging

import pandas as pd
from autogluon.tabular import TabularPredictor

from autoconf.utils.pydantic_models import JobConfig
from autoconf.utils.rule_based_classifier import is_row_valid

# Configure logging

logging.basicConfig(
    level=logging.INFO,  # Set the logging level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",  # Log format
    handlers=[logging.StreamHandler()],  # Output to console
)

logger = logging.getLogger(__name__)

# NOTE: this list will not be used if the user provides them

VALID_N_GPUS = [1, 2, 4, 8, 16, 32]


def get_model_prediction_and_metadata(
    config: pd.DataFrame | dict | JobConfig, predictor
) -> tuple[int, dict]:
    """Gets valid/invalid prediction and reason why"""
    if isinstance(config, dict):
        config = pd.DataFrame(config, index=[0])
    if isinstance(config, JobConfig):
        config = pd.DataFrame([config.model_dump()], index=[0])

    metadata = {}
    pred = None
    machine_learning_classifier_error = None
    row_is_valid, rule_based_classifier_error = is_row_valid(config)
    if int(row_is_valid) == 1:
        try:
            pred = predictor.predict(config).values[0]
            logger.debug("Prediction succeeded")
        except Exception as e:
            logger.debug("Prediction FAILED")
            machine_learning_classifier_error = str(e)

    metadata["Rule-Based Classifier error"] = " ".join(rule_based_classifier_error)
    metadata["Predictive Model Classifier error"] = machine_learning_classifier_error

    # NOTE:  is added here to account for invalid by rule based classifier
    pred = int(pred) if pred else 0
    return pred, metadata


def recommend_min_gpu(
    job_config: JobConfig, predictor, valid_n_gpu_list: list[int] | None = None
) -> tuple[int, dict]:
    """Recommends the minimum number of GPUs required for a SFT job defined by the fields of the pydantic model :job_config:
    Returns
        min_n_gpu: the minimum number of valid gpus
        -1 if no gpu number in the valid_n_gpu list is predicted to be valid"""
    # We use list() here to copy VALID_N_GPUS and thus avoid any unintentional update to VALID_N_GPUS.
    if valid_n_gpu_list is None:
        valid_n_gpu_list = list(VALID_N_GPUS)

    if isinstance(predictor, str):
        predictor = TabularPredictor.load(predictor, require_py_version_match=False)

    metadata = {"default": "User config was not provided"}

    # VV: Find the minimum number of GPUs that the recommender predicts will successfully run this tuning job.
    # Start from the lowest candidate and stop when the recommender predicts that the workload will successfully
    # run with that GPU count.

    min_number_gpus = -1

    valid_n_gpu_list = sorted(valid_n_gpu_list)
    for candidate_number_gpus in valid_n_gpu_list:
        logger.info(f"Testing number_gpus={candidate_number_gpus}")
        if job_config.number_gpus and candidate_number_gpus == job_config.number_gpus:
            logger.info(
                "This is the value provided by the user, for this configuration the recommender will provide additional metadata"
            )
            gpus_can_support_run, _ = get_model_prediction_and_metadata(
                job_config, predictor=predictor
            )
        else:
            new_job_config = job_config.model_copy(
                update={"number_gpus": candidate_number_gpus}
            )
            gpus_can_support_run, metadata = get_model_prediction_and_metadata(
                new_job_config, predictor=predictor
            )

        logger.info(
            f"Prediction for ngpu={candidate_number_gpus}\t:\t{gpus_can_support_run}\t(note:0 is not valid, 1 is Valid)"
        )

        if gpus_can_support_run == 1:
            min_number_gpus = candidate_number_gpus
            break

    logger.info(
        f"""Metadata related to the model prediction
        (number_gpus={job_config.number_gpus if job_config.number_gpus else 'Not provided'})
        :{metadata}"""
    )

    if min_number_gpus == -1:
        logger.info(
            f"""A recommendation for 'number_gpus' cannot be provided because
            no values for 'number_gpus' of the list {valid_n_gpu_list} would result
            in a valid run according to the predictive model."""
        )
    else:
        logger.info(f"The recommended number_gpus={min_number_gpus}.")

    return min_number_gpus, metadata


class MinGpuRecommender:
    def __init__(self, predictor, valid_n_gpu: list[int] | None = None):
        self.valid_n_gpu = valid_n_gpu or list(VALID_N_GPUS)

        if isinstance(predictor, str):
            self.predictor = TabularPredictor.load(
                predictor, require_py_version_match=False
            )
        else:
            self.predictor = predictor

    def recommend_min_gpu(self, job_config):
        return recommend_min_gpu(job_config, self.predictor, self.valid_n_gpu)

    def predict(self, job_config: JobConfig | pd.DataFrame):
        if isinstance(job_config, pd.DataFrame):
            # Convert DataFrame rows to JobConfig instances
            job_configs = [
                JobConfig.model_validate(row.dropna().to_dict())
                for _, row in job_config.iterrows()
            ]
        else:
            job_configs = [job_config]

        # Run prediction for each config
        return [
            recommend_min_gpu(config, self.predictor, self.valid_n_gpu)[0]
            for config in job_configs
        ]


class NoRecommendationError(ValueError):
    def __init__(self, reason: str):
        self.reason = reason

    def __str__(self):
        return f"Unable to recommend minimum number of GPUs to avoid GPU OOM: {self.reason}"



# Copyright (c) IBM Corporation

# SPDX-License-Identifier: MIT

# %% Run this script with IPython
import glob
import logging
import os
import shutil

import pandas as pd
from autogluon.tabular import TabularDataset, TabularPredictor

from autoconf.utils.rule_based_classifier import is_row_valid

logger = logging.getLogger(__name__)
logger.info("These are the available csvs")
glob.glob("*", root_dir="../../../data")  # %%

# %%
REFIT = True

d_s = "11-13-dashboard-163-for-min-gpu"
path = f"../../../data/{d_s}.csv"
df_original = pd.read_csv(path)
clist = list(df_original.columns)
cols_to_use = [
    "model_name",
    "method",  # LoRA, FULL
    "number_gpus",
    "gpu_model",
    "tokens_per_sample",  # this is: max_sequence_lenght
    "batch_size",
    "is_valid",  # Has the job being successful or did it have OOM problems?
    # NOTE: jobs that are not successful for incorrect specification of the config file are filtered out before training the model.
]
logger.info(set(df_original["model_name"].values))

# %%
fit_params = {"presets": ["medium_quality"], "excluded_model_types": "GBM"}
target = "is_valid"


def filter_valid_with_hard_logic(df: pd.DataFrame):
    logger.debug(f"l before {len(df)}")
    valid_indices = [i for i, config in df.iterrows() if is_row_valid(config)[0]]
    df_filtered = df.loc[valid_indices].copy()
    logger.debug(f"l after {len(df_filtered)}")
    return df_filtered


# Our default is filtering valid rows with hard logic first
df = filter_valid_with_hard_logic(df_original)
df = df.sample(frac=1).reset_index(drop=True)


# %% You can decide here if you want to train
train_fraction = 0.8
train_idx = int(len(df) * train_fraction)
df_train = df.iloc[:train_idx][cols_to_use]
df_test = df.iloc[train_idx:][cols_to_use]

df_test = filter_valid_with_hard_logic(df_test)

# %% TRAIN
train_data = TabularDataset(df_train)
train_data.head()
predictor = TabularPredictor(label=target).fit(train_data, **fit_params)
model_path = predictor.path
size_original = predictor.disk_usage()
logger.info("Model path is: ", model_path)

# %% TEST
test_data = TabularDataset(df_test)
y_pred = predictor.predict(test_data.drop(columns=[target]))

d = predictor.evaluate(test_data, silent=True)
d_name = [predictor.eval_metric.name]
logger.info("Evaluation is", d)


# %% Refitting the original model is discretionary,  it improves inference speed but diminishes accuracy
# docs at <https://auto.gluon.ai/stable/api/autogluon.tabular.TabularPredictor.html>
if REFIT:
    predictor.refit_full(model="best", set_best_to_refit_full=True)


save_path_refit_clone_opt = model_path + "-refit-clone-opt"
path_clone_opt = predictor.clone_for_deployment(path=save_path_refit_clone_opt)
predictor_clone_opt = TabularPredictor.load(path=save_path_refit_clone_opt)

# %% Logging size comparison
size_refit_opt = predictor_clone_opt.disk_usage()
logger.info(f"Size Original:  {size_original} bytes")
logger.info(f"Size Optimized: {size_refit_opt} bytes")
logger.info(
    f"Optimized predictor achieved a {round((1 - (size_refit_opt/size_original)) * 100, 1)}% reduction in disk usage."
)

predictor_clone_opt.evaluate(test_data, silent=True)
# %% cleaning up files, keeping only the refit-opt model
if model_path and os.path.isdir(model_path):
    try:
        shutil.rmtree(model_path, ignore_errors=True)
        logger.info(f"Deleted model directory: {model_path}")
    except Exception as e:
        logger.info(f"Could not delete model directory '{model_path}': {e}")



# Copyright (c) IBM Corporation
# SPDX-License-Identifier: MIT

from typing import Annotated

from pydantic import BaseModel, BeforeValidator, Field

from autoconf.utils.config_mapper import map_valid_model_name


class JobConfig(BaseModel):
    model_name: Annotated[str, BeforeValidator(map_valid_model_name)]
    method: str
    gpu_model: str
    tokens_per_sample: int = Field(..., ge=1, description="Max sequence length")
    batch_size: int = Field(..., ge=1)
    is_valid: int | None = Field(
        default=None,
        description="Ground truth. 1 if job was successful. It is not used for prediction purposes",
    )
    number_gpus: int | None = Field(
        default=None, ge=1, description="Number of GPUs used"
    )
